@@ -14,6 +14,7 @@
  *
  */
 #define _GNU_SOURCE
+#include <libgen.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
@@ -24,6 +25,7 @@
 #include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/random.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <linux/fb.h>
@@ -328,12 +330,14 @@ static int add_edd_entry(struct x86_linux_param_header *real_mode,
 	memset(edd_info, 0, sizeof(struct edd_info));
 
 	/* extract the device number */
-	if (sscanf(basename(sysfs_name), "int13_dev%hhx", &devnum) != 1) {
+	char* sysfs_name_copy = strdup(sysfs_name);
+	if (sscanf(basename(sysfs_name_copy), "int13_dev%hhx", &devnum) != 1) {
 		fprintf(stderr, "Invalid format of int13_dev dir "
-				"entry: %s\n", basename(sysfs_name));
+				"entry: %s\n", basename(sysfs_name_copy));
+		free(sysfs_name_copy);
 		return -1;
 	}
-
+	free(sysfs_name_copy);
 	/* if there's a MBR signature, then add it */
 	if (file_scanf(sysfs_name, "mbr_signature", "0x%x", &mbr_sig) == 1) {
 		real_mode->edd_mbr_sig_buffer[*current_mbr] = mbr_sig;
@@ -544,6 +548,7 @@ struct setup_data {
 #define SETUP_DTB	2
 #define SETUP_PCI	3
 #define SETUP_EFI	4
+#define SETUP_RNG_SEED	9
 	uint32_t len;
 	uint8_t data[0];
 } __attribute__((packed));
@@ -824,6 +829,26 @@ static void setup_e820(struct kexec_info *info, struct x86_linux_param_header *r
 	}
 }
 
+static void setup_rng_seed(struct kexec_info *info,
+			   struct x86_linux_param_header *real_mode)
+{
+	struct {
+		struct setup_data header;
+		uint8_t rng_seed[32];
+	} *sd;
+
+	sd = xmalloc(sizeof(*sd));
+	sd->header.next = 0;
+	sd->header.len = sizeof(sd->rng_seed);
+	sd->header.type = SETUP_RNG_SEED;
+
+	if (getrandom(sd->rng_seed, sizeof(sd->rng_seed), GRND_NONBLOCK) !=
+	    sizeof(sd->rng_seed))
+		return; /* Not initialized, so don't pass a seed. */
+
+	add_setup_data(info, real_mode, &sd->header);
+}
+
 static int
 get_efi_mem_desc_version(struct x86_linux_param_header *real_mode)
 {
@@ -923,9 +948,27 @@ void setup_linux_system_parameters(struct kexec_info *info,
 
 	setup_e820(info, real_mode);
 
+	/* pass RNG seed */
+	setup_rng_seed(info, real_mode);
+
 	/* fill the EDD information */
 	setup_edd_info(real_mode);
 
 	/* Always try to fill acpi_rsdp_addr */
 	real_mode->acpi_rsdp_addr = get_acpi_rsdp();
 }
+
+void setup_linux_dtb(struct kexec_info *info, struct x86_linux_param_header *real_mode,
+				   const char *dtb_buf, int dtb_len)
+{
+	struct setup_data *sd;
+
+	sd = xmalloc(sizeof(struct setup_data) + dtb_len);
+	sd->next = 0;
+	sd->len = dtb_len;
+	sd->type = SETUP_DTB;
+	memcpy(sd->data, dtb_buf, dtb_len);
+
+
+	add_setup_data(info, real_mode, sd);
+}
