@@ -701,10 +701,13 @@ static void update_purgatory(struct kexec_info *info)
 			continue;
 		}
 
-		/* Don't include elfcorehdr in the checksum, if hotplug
-		 * support enabled.
+		/*
+		 * Let architecture decide which segments to exclude from checksum
+		 * if hotplug support is enabled.
 		 */
-		if (do_hotplug && (info->segment[i].mem == (void *)info->elfcorehdr)) {
+		if (do_hotplug && arch_do_exclude_segment(info, &info->segment[i])) {
+			dbgprintf("Skipping segment mem: 0x%lx from SHA calculation\n",
+				  (unsigned long)info->segment[i].mem);
 			continue;
 		}
 
@@ -1090,7 +1093,9 @@ void usage(void)
 	       "                      back to the compatibility syscall when file based\n"
 	       "                      syscall is not supported or the kernel did not\n"
 	       "                      understand the image (default)\n"
-	       " --hotplug            Setup for kernel modification of elfcorehdr.\n"
+	       " --hotplug            Do in-kernel update of kexec segments on CPU/Memory\n"
+	       "                      hot add/remove events, avoiding the need to reload\n"
+	       "                      kdump kernel on online/offline events.\n"
 	       " -d, --debug          Enable debugging to help spot a failure.\n"
 	       " -S, --status         Return 1 if the type (by default crash) is loaded,\n"
 	       "                      0 if not.\n"
@@ -1419,6 +1424,30 @@ static int do_kexec_file_load(int fileind, int argc, char **argv,
 	return ret;
 }
 
+/*
+ * Hotplug support for x86 in the kernel was added with the
+ * `KEXEC_UPDATE_ELFCOREHDR` kexec bit, and later it was generalized
+ * with the `KEXEC_CRASH_HOTPLUG_SUPPORT` kexec bit.
+ *
+ * Passing the `KEXEC_CRASH_HOTPLUG_SUPPORT` kexec bit to kernel
+ * versions 6.5 to 6.9 on x86 with the `kexec_load` system call will
+ * fail with `-EINVAL`.
+ *
+ * So for now, pass `KEXEC_UPDATE_ELFCOREHDR` for x86, and for other
+ * architectures, use the `KEXEC_CRASH_HOTPLUG_SUPPORT` kexec bit. But
+ * in the future, we can decide to get rid of `KEXEC_UPDATE_ELFCOREHDR`.
+ *
+ * NOTE: Xen KEXEC_LIVE_UPDATE and KEXEC_UPDATE_ELFCOREHDR collide
+ */
+static inline unsigned long get_hotplug_kexec_flag(void)
+{
+#if defined(__i386__) || defined(__x86_64__)
+		return KEXEC_UPDATE_ELFCOREHDR;
+#else
+		return KEXEC_CRASH_HOTPLUG_SUPPORT;
+#endif
+}
+
 static void print_crashkernel_region_size(void)
 {
 	uint64_t start = 0, end = 0;
@@ -1477,6 +1506,7 @@ int main(int argc, char *argv[])
 			return 0;
 		case OPT_DEBUG:
 			kexec_debug = 1;
+			kexec_file_flags |= KEXEC_FILE_DEBUG;
 			break;
 		case OPT_NOIFDOWN:
 			skip_ifdown = 1;
@@ -1650,7 +1680,6 @@ int main(int argc, char *argv[])
 		die("--load-live-update can only be used with xen\n");
 	}
 
-	/* NOTE: Xen KEXEC_LIVE_UPDATE and KEXEC_UPDATE_ELFCOREHDR collide */
 	if (do_hotplug) {
 		const char *ces = "/sys/kernel/crash_elfcorehdr_size";
 		char *buf, *endptr = NULL;
@@ -1664,8 +1693,10 @@ int main(int argc, char *argv[])
 		if (!elfcorehdrsz || (endptr && *endptr != '\0'))
 			die("Path %s does not exist, the kernel needs CONFIG_CRASH_HOTPLUG\n", ces);
 		dbgprintf("ELFCOREHDR_SIZE %lu\n", elfcorehdrsz);
-		/* Indicate to the kernel it is ok to modify the elfcorehdr */
-		kexec_flags |= KEXEC_UPDATE_ELFCOREHDR;
+		/* Indicate to the kernel it is ok to modify the relevant kexec segments */
+
+		kexec_flags |= get_hotplug_kexec_flag();
+
 	}
 
 	fileind = optind;
@@ -1685,6 +1716,10 @@ int main(int argc, char *argv[])
 			}
 		}
 	}
+	if (xen_present()) {
+		do_kexec_file_syscall = 0;
+		do_kexec_fallback = 0;
+	}
 	if (do_kexec_file_syscall) {
 		if (do_load_jump_back_helper && !do_kexec_fallback)
 			die("--load-jump-back-helper not supported with kexec_file_load\n");
